package BrowserEdit;

use strict;
use vars qw(@ISA);
use CGI::XMLApplication;
use XML::LibXML;
use XML::Schematron::LibXSLT;

@ISA = qw(CGI::XMLApplication);

# event callbacks
sub registerEvents { 
    qw( browseredit_edit browseredit_save); 
}

sub event_init {
    my ($self, $context) = @_;
    # internal init.
    $context->{ERRORS} = [];
    $context->{SCHEMA} ||= '/usr/local/apache/cgi-bin/xmldocs/webcontent.scm';

    if ( defined $self->param('xml_content') ) {
        $context->{CONTENT} = $self->param('xml_content');
    }
    elsif ( defined $self->param('file_name') ) {
        my $path = $self->param('file_name');
        
        eval {
           open( XML, $path) || die $!;
        };
        if ( $@ ) {
            my $error = "Could not open file '$path' for reading: $@\n";
            $context->{CONTENT} = '';
            push @{$context->{ERRORS}}, $error;
            $self->sendEvent( 'default' );
        }
        else {
            local $/;
            my $content = <XML>;
            close XML;

            $context->{CONTENT} = $content;
            $self->param(-name => 'xml_content', -value => $content);
        }
    }            
}

sub event_default {
    my ($self, $context) = @_;
    $context->{SCREENSTYLE} = 'browseredit_default.xsl';
}

sub event_browseredit_edit {
    my ($self, $context) = @_;
    $self->verify_content( $context );
    $context->{SCREENSTYLE} = 'browseredit_edit.xsl';
}

sub event_browseredit_save {
    my ($self, $context) = @_;
    $self->verify_content( $context );
    if ( scalar( @{$context->{ERRORS}} ) == 0 ) {
        my $file = $self->param( 'file_name' );

        eval {
           open( XML, ">$file") || die $!;
        };
        if ( $@ ) {
            my $error = "Could not open file '$file' for writing: $@\n";
            push @{$context->{ERRORS}}, $error;
        }
        else {
            print XML $context->{CONTENT};
            close XML;
        }        
    }
    $context->{SCREENSTYLE} = 'browseredit_save.xsl';
}


# app config and helpers
sub selectStylesheet {
    my ($self, $context) = @_;
    my $style = $context->{SCREENSTYLE} || 'default';
    my $style_path = '/usr/local/apache/cgi-bin/stylesheets/';
    return $style_path . $style;
}
 
sub requestDOM {
    my ($self, $context) = @_;
    my $dom = XML::LibXML::Document->new();
    my $root = $dom->createElement( 'document' );
    $dom->setDocumentElement( $root );
   
    # add errors if any
    if ( scalar( @{$context->{ERRORS}} ) > 0 ) {
        my $errors = $dom->createElement( 'errors' );

        foreach my $message ( @{$context->{ERRORS}} ) {
            $errors->appendTextChild( 'error', $message );
        }

        $root->appendChild( $errors );
    }

    return $dom;
}

sub getXSLParameter {
    my ( $self, $context ) = @_;
    return %{ $self->Vars() };
}

sub verify_content {
    my ( $self, $context ) = @_;
    my $content = $context->{CONTENT};
    warn "content $content \n";
    my $tron = XML::Schematron::LibXSLT->new( );
    $tron->schema( $context->{SCHEMA} );
    my @messages = ();

    eval {
        @messages = $tron->verify( $content );
    };
    if ( $@ ) {
        my $error = "Error processing XML document: $@";
        push @{$context->{ERRORS}}, $error;
    }
    else {
        push @{$context->{ERRORS}}, @messages;
    }
} 

1;
